/*
 * just a test to see if tclobj is working
 */

#include <stdlib.h>
#include <tcl.h>
#include <string.h>
#include <tclobj.h>
#include "testobj.h"

static const TclObj_Comtab testtable[] = {
  /* constructors */
  { 1, "new", "",      "creates empty string object" },
  { 2, "new", "test",  "copy constructor" },

  /* operators */
  { 1, "set", "...",   "set value of object"             },
  { 2, "get", "",      "query value of object"           },
  { 3, "add", "...",   "append text to string"           },
  { 4, "copy","test",  "copy contents of other object"   },
  { 5, "dup", "",      "create new object with our data" },
  { 0, NULL, NULL, NULL }
};

TCL_OBJECT_DECL_BODY (test,Tcl_Object,testtable,NULL);

test::test (Tcl_Interp *interp, int argc, char *argv[], int command)
{
  test *other;

  Tcl_DStringInit (&mystring);

  switch (command) {
  case 2:
    other = TCL_OBJECT_ARG_OBJECT(interp, argv[2], test);
    Tcl_DStringAppend (&mystring,
		       Tcl_DStringValue (&(other->mystring)),
		       -1);
    break;
  }
}

test::test (test &other)
{
  Tcl_DStringInit   (&mystring);
  Tcl_DStringAppend (&mystring,
		     Tcl_DStringValue (&(other.mystring)), -1);
}

test::~test ()
{
  Tcl_DStringFree (&mystring);
}

int
test::Tcl_Proc (Tcl_Interp *interp,
		int argc, char *argv[],
		int command)
{
  test *other;
  int index;

  switch (command) {
  case 1: /* set */
    Tcl_DStringSetLength (&mystring, 0);
    for (index=2; index<argc; index++) {
      Tcl_DStringAppend (&mystring, argv[index], -1);
      if (index+1<argc)
	Tcl_DStringAppend (&mystring, " ", 1);
    }
    Tcl_SetResult (interp, Tcl_DStringValue (&mystring), TCL_VOLATILE);
    break;
  case 2: /* get */
    Tcl_SetResult (interp, Tcl_DStringValue (&mystring), TCL_VOLATILE);
    break;
  case 3: /* add */
    for (index=2; index<argc; index++) {
      Tcl_DStringAppend (&mystring, " ", 1);
      Tcl_DStringAppend (&mystring, argv[index], -1);
    }
    Tcl_SetResult (interp, Tcl_DStringValue (&mystring), TCL_VOLATILE);
    break;
  case 4: /* copy */
    other = TCL_OBJECT_ARG_OBJECT(interp, argv[2], test);
    Tcl_DStringSetLength (&mystring, 0);
    Tcl_DStringAppend    (&mystring, 
			  Tcl_DStringValue (&(other->mystring)), -1);
    Tcl_SetResult (interp, Tcl_DStringValue (&mystring), TCL_VOLATILE);
    break;
  case 5: /* dup */
    other = new test(*this);
    return TCL_OBJECT_RETURN_OBJECT(interp, other);
    break;
  default:
    TCL_OBJECT_NO_MATCH(interp);
  }
  return TCL_OK;
}
